#!/usr/bin/env python
#
# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2016 USIT-University of Oslo
#
# zabbix_elasticsearch_cluster_health.py: Used by zabbix_agent to pull
# elasticsearch cluster health information from an ES cluster and send
# this information to Zabbix via trappers.
#
# zabbix_elasticsearch_cluster_health.py is free software: you can
# redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# zabbix_elasticsearch_cluster_health.py is distributed in the hope
# that it will be useful, but WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with sms_send.  If not, see <http://www.gnu.org/licenses/>.
#

#
# This script gets ES cluster health information from the ES rest API,
# extracts the parameters we have defined in health_keys{} and sends
# data back to zabbix via zabbix_sender to defined trap-items.
#
# The script is executed via zabbix_agent and is defined in an
# UserParameter that will return 0 (execution OK) or 1 (execution
# ERROR) so zabbix can register if it cannot get data from ES.
#
# UserParameter=get.es.cluster.health[*],/usr/bin/zabbix_elasticsearch_cluster_health.py $1
#

import requests
import json
import sys
import os
import time
import tempfile

# Zabbix proxy
zabbix_proxy = ['zabbix-proxy-prod01.uio.no','zabbix-proxy-prod02.uio.no']

# Path to zabbix_sender
zabbix_sender = '/usr/bin/zabbix_sender'

# Temp file with full json output
tmp_stat_file = tempfile.NamedTemporaryFile(delete=False,dir='/tmp')


# Item prefix
item_prefix = 'es.cluster.'

# keys for health page
health_keys = {          
    "cluster_name",
    "status",
    "timed_out",
    "number_of_nodes",
    "number_of_data_nodes",
    "active_primary_shards",
    "active_shards",
    "relocating_shards",
    "initializing_shards",
    "unassigned_shards",
    "delayed_unassigned_shards",
    "number_of_pending_tasks",
    "number_of_in_flight_fetch",
    "task_max_waiting_in_queue_millis",
    "active_shards_percent_as_number"
}

# ############################################
# getKeys()
# ############################################

def getKeys(json_data,keys,node_fqdn):
    result=''
    
    for key in keys:
        attributes=key.split('.')
        value=json_data

        for index in range(len(attributes)):
            value=value.get(attributes.pop(0),{})

            if value=={}:
                continue

        result += node_fqdn + ' ' + item_prefix + "{0} {1}\n".format(key,value)

    return result


# ############################################
# get_cluster_health_data()
# ############################################

def get_cluster_health_data(node_fqdn):
    """
    Get ES cluster health data
    """


    try:
        request_data = requests.get("http://" + node_fqdn + ":9200/_cluster/health")
        health_data = request_data.json()

        result = getKeys(health_data,health_keys,node_fqdn)

    except Exception, e:
        print "1"
        sys.exit(1)

    return result

# ############################################
# Main
# ############################################

if __name__ == '__main__':

    try:

        if len(sys.argv) == 2:
            
            node_fqdn = sys.argv[1].lower()
            result = get_cluster_health_data(node_fqdn)

            #
            # We create a file with the data that zabbix_sender will
            # send in a bulk execution.
            #

            with open(tmp_stat_file.name,'w') as f: 
                f.write(result)
        
            #
            # The monitoring of this host can be done by any of the
            # zabbix proxyer defined in zabbix_proxy[]. We try all of
            # them until one of them accepts our data
            #

            for proxy in zabbix_proxy:
                returncode = os.system(zabbix_sender + ' -z ' + proxy + ' -i ' + tmp_stat_file.name + ' > /dev/null 2>&1')

                if returncode == 0:
                    break

            if returncode != 0:
                print "1"
                sys.exit(1)
            
        else:
            print "1"
            sys.exit(1)

    except Exception, e:
        print "1"
        sys.exit(1)

    # Delete temp file with zabbix_sender data
    os.remove(tmp_stat_file.name)

    # Return value 0 = execution OK
    print "0"
